#!/usr/bin/env python3

"""
This module provides a function to extract settings saved by Novelda HPD Studio.
"""

import json
import os
import platform
import re
import tempfile


def get_settings(pipe_name=''):
    r"""
    Makes full path to named pipe name on Windows (\\.\pipe\hpd is the default), on other OS's use user
    provided path for FIFO (or use the default one: "/tmp/hpd.fifo").
    Extracts settings saved by Novelda HPD Studio. Streaming option should be enabled in Novelda HPD Studio.

    Args:
        pipe_name (str): name of pipe/FIFO.Change

    Returns:
        A dictionary with at least the following fields:
        - **timeout_period** (*int*): 'presence hold' timeout. Check Novelda HPD Studio for details.
        - **maxrange_cm** (*str*): detection range (if supported by algorithm).
        - **selected_sensitivity** (*str*): selected sensitivity (if supported by algorithm).
        - **fifo_path** (*str*): path to named pipe/FIFO.

    Raises:
        FileNotFoundError: If pipe/FIFO or settings file couldn't be open.
    """

    settings_file_path = os.path.join(tempfile.gettempdir(), 'selected_options.json')

    try:
        with open(settings_file_path, 'r', encoding='UTF-8') as file:
            _settings = json.load(file)
    except FileNotFoundError as err:
        raise FileNotFoundError(f'File "{settings_file_path}" does not exist.\n'
                                'Start Novelda HPD Studio with the "-p" argument or enable streaming to pipe in GUI.') \
                                from err

    if platform.system() == 'Windows':
        _settings['fifo_path'] = f'\\\\.\\pipe\\{pipe_name}' if pipe_name else '\\\\.\\pipe\\hpd'
    else:
        _settings['fifo_path'] = pipe_name if pipe_name else '/tmp/hpd.fifo'

    if 'timeout_period' in _settings:
        _settings['timeout_period'] = int(_settings['timeout'])
    else:
        _settings['timeout_period'] = 0

    if 'range' in _settings:
        _settings['maxrange_cm'] = re.findall(r'\d+', os.path.splitext(os.path.split(_settings['range'])[1])[0])[-1]
    else:
        _settings['maxrange_cm'] = 0

    if 'selected_sensitivity' in _settings:
        _settings['selected_sensitivity'] = re.findall(r'\d+', os.path.splitext(os.path.split(_settings['sensitivity'])[1])[0])[-1]
    else:
        _settings['selected_sensitivity'] = 0

    return _settings


if __name__ == '__main__':
    settings = get_settings()
    for key in settings:
        print(f'{key}: {settings[key]}')
