#!/usr/bin/env python3

"""
This demo displays the same information as 'print_distance.py', but does so with a GUI.
"""

import sys
import signal
from PySide2 import QtCore, QtWidgets, QtGui
import signal_frame_utils as sfu
import x4_utils as x4
# pylint: disable=invalid-name


class PresenceData(QtCore.QObject):
    """
    Keeps track of the detection data, in particular the
    distance between the sensor and the nearest detected object.
    """

    distance = QtCore.Signal(int)


class ReadThread(QtCore.QThread):
    """
    Logic for reading the signal frame and updating the
    detected distance.
    """

    def __init__(self, parent=None):
        QtCore.QThread.__init__(self, parent)
        self.presenceData = PresenceData()

    def applicationLogic(self, data):
        """
        Updates the detection distance for each frame.

        Args:
            data: SignalFrame object (e.g. extracted by `read_signal_stream` function)

        Returns:
        """

        power = x4.get_power(data)
        if power is not None:   # not every signal_frame contains power data
            distance = x4.get_distance(power)
            self.presenceData.distance.emit(distance)

    def run(self):
        """
        Reads the signal stream.

        Returns: None
        """

        sfu.read_signal_stream(self.applicationLogic)


class DistanceWindow(QtWidgets.QMainWindow):
    """
    Sets up the GUI displaying the distance, and updates the distance
    estimate whenever the detected distance changes, if no object is detected
    it displays 'NO PRESENCE'.
    """

    def __init__(self, parent=None):
        QtWidgets.QMainWindow.__init__(self, parent)
        self.setWindowTitle('Novelda Distance Demo')
        self.setStyleSheet('QFrame {background-color: rgb(2,66,84); color: rgb(23,211,226); }')
        self.label = QtWidgets.QLabel('NO PRESENCE')
        self.label.setFont(QtGui.QFont('Arial', 40))
        self.label.setAlignment(QtCore.Qt.AlignCenter)
        self.setCentralWidget(self.label)

        QtWidgets.QApplication.instance().readThread.presenceData.distance.connect(self.newData)
        self.show()

    def newData(self, data):
        """
        Updates the data in the GUI.

        Args:
            data (int): estimate for distance between sensor and closest
                        detected object

        Returns: None
        """

        self.label.setText(f'DISTANCE {data}' if data else 'NO PRESENCE')


class Application(QtWidgets.QApplication):
    """
    Initializes the application and reads sensor input.
    """

    def __init__(self, params=None):
        QtWidgets.QApplication.__init__(self, params)
        self.aboutToQuit.connect(self.cleanUp)
        self.readThread = ReadThread(self)
        self.readThread.finished.connect(self.quit)
        QtCore.QTimer.singleShot(0, self.readThread.start)

    def cleanUp(self):
        """
        Cleans and halts further reading before exiting the application.

        Returns: None
        """

        self.readThread.terminate()
        self.readThread.wait(1000)
        sys.exit(0)


# pylint: disable=unused-argument
def sigintHandler(*args):
    """
    Signal handler used in signal.signal

    Args:
        *args:
            arg1: the signal number
            arg2: current stack frame

    Returns: None
    """

    QtWidgets.QApplication.quit()
# pylint: enable=unused-argument


if __name__ == '__main__':
    app = Application(sys.argv)
    signal.signal(signal.SIGINT, sigintHandler)
    distanceWindow = DistanceWindow()
    sys.exit(app.exec_())
