#!/usr/bin/env python3

"""
This module provides functions to extract data from SignalFrame object generated by SignalFlow.
"""

import collections
from typing import List
import numpy as np
from fnvhash import fnv1a_32
from Novelda.Serialization.SignalFrame import SignalFrame
from hpd_studio_utils import get_settings


def get_array(signal, supported_semantics: List[bytes]):
    if len(supported_semantics) == 0:
        return None
    supported_arrays_hash = [fnv1a_32(s) for s in supported_semantics]
    for i in range(0, signal.ArraysLength()):
        if signal.Arrays(i).ArraySemantic() in supported_arrays_hash:
            return signal.Arrays(i).Array()
    return None


def _check_node_name(signal, node_names: List[bytes] = None):
    if node_names is None:
        node_names = []
    if len(node_names) == 0:
        return True
    array = get_array(signal, [b'node_name'])
    supported_nodes_hash = [fnv1a_32(s) for s in node_names]
    return array.DataUInt32AsNumpy()[0] in supported_nodes_hash if array else False


def get_supported_signals(signal_frame, supported_signals, node_name: List[bytes] = None):
    if node_name is None:
        node_name = []
    supported_signals_hash = [fnv1a_32(s) for s in supported_signals]
    signals = []
    for i in range(signal_frame.SignalsLength()):
        signal = signal_frame.Signals(i)
        if signal.SignalSemantic() in supported_signals_hash and _check_node_name(signal, node_name):
            signals.append(signal_frame.Signals(i))
    return signals


def read_signal_stream(callback, fifo_path=None):
    """
    Extracts SignalFrame object generated by SignalFlow from named pipe or FIFO and calls callback function.

    Novelda HPD Studio can stream raw data generated by SignalFlow into named pipe on Windows
    or FIFO on Unix-like systems. This function reads this data and calls callback function to
    process extracted SignalFrame object. If algorithm runs at 1 FPS callback is called once each second.
    For GUI applications it is a good practice to run this function in a separate thread.

    Args:
        callback: callable object to process extracted SignalFrame object, e.g. `get_power`.
        fifo_path:

    Returns: None
    """

    if fifo_path is None:
        fifo_path = get_settings()['fifo_path']

    try:
        with open(fifo_path, 'rb') as fifo:
            while fifo:
                prefix_size = 4
                data = fifo.read(prefix_size)
                read_size = len(data)
                if read_size != prefix_size:
                    return      # FIFO most likely closed.
                frame_size = int.from_bytes(data, byteorder='little', signed=False)
                frame_data = fifo.read(frame_size)
                read_size = len(frame_data)
                if read_size < frame_size:
                    print(f'Partial frame received, expected {frame_size} got {read_size}.')
                else:
                    callback(SignalFrame.GetRootAsSignalFrame(frame_data, 0))

    except FileNotFoundError:
        print(f'Could not open "{fifo_path}".\n'
              'Start Novelda HPD Studio with the "-p" argument or enable streaming to pipe in GUI.')
    except KeyboardInterrupt:
        fifo.close()
    except OSError as err:
        if err.errno == 22:
            print('Pipe closed by writer.')
