#!/usr/bin/env python3

"""
This demo prints whenever the sensor's detection status changes, when it does the following output
is written to the terminal:
<int: unix timestamp> <string: detection status>
where 'detection status' is either 'PRESENCE' or 'NO PRESENCE'.
"""

import functools
import time
import signal_frame_utils as sfu
import x4_utils as x4

print = functools.partial(print, flush=True)

class PresenceState:
    """
    Holds information regarding previous and current detection status
    """

    STATE_TEXT = ['NO PRESENCE', 'PRESENCE']

    def __init__(self):
        self.current_presence = None
        self.last_presence = False

    def state_change(self):
        """
        Checks if the current detection state is different from the last.

        Returns: Boolean
        """
        return self.current_presence != self.last_presence

    def print_status(self):
        """
        Print unix timestamp as well as the current detection status;
        it does so in the following format:
        <int: unix timestamp> <string: detection status>
        where 'detection status' is either 'PRESENCE' or 'NO PRESENCE'

        Returns: None
        """

        print(f'{int(time.time())} {self.STATE_TEXT[int(self.current_presence)]}')


def update_presence(signal_frame):
    """
    Keeps track of if the sensor is currently detecting anything, and whenever the
    detection status changes it prints a Unix timestamp as well as the current detection status;
    it does so in this format:
    <int: unix timestamp> <string: detection status>
    where 'detection status' is either 'PRESENCE' or 'NO PRESENCE'

    Args:
        signal_frame: SignalFrame object (e.g. extracted by `read_signal_stream` function)

    Returns: None
    """

    presence = x4.get_presence(signal_frame)
    if presence is not None:    # Not every frame contains data
        presence_state.current_presence = presence['detection']
        if presence_state.state_change():
            presence_state.print_status()
            presence_state.last_presence = presence_state.current_presence


if __name__ == '__main__':
    presence_state = PresenceState()
    sfu.read_signal_stream(update_presence)
