#!/usr/bin/env python3

"""
This module provides functions to extract data from SignalFrame object generated by SignalFlow X4 flows.
"""

import collections
import numpy as np
import signal_frame_utils as sfu


def get_presence(signal_frame):
    """
    Extracts sequence number and presence from SignalFrame.

    Extracts sequence number and presence from SignalFrame generated by
    X4Sensor node (e.g. Occupancy and Proximity flows).

    Args:
        signal_frame: SignalFrame object (e.g. extracted by `read_signal_stream` function).

    Returns:
        A dictionary with the following fields:
        - **sequence_number** (*int*): sequence number of the input signal frame object.
        - **detection** (*bool*): detection/no detection.
    """

    for signal in sfu.get_supported_signals(signal_frame, [b'human_presence'], [b'X4Sensor']):
        presence_array = sfu.get_array(signal, [b'human_presence_basic'])
        if presence_array:
            return {
                'sequence_number': int(signal_frame.SequenceNumber()),
                'detection': bool(presence_array.DataInt32AsNumpy()[0]),
            }
    return None


# pylint: disable=too-many-locals
def get_power(signal_frame):
    """
    Extracts power and threshold data from SignalFrame.

    Extracts power and threshold data from SignalFrame generated by detector nodelet for
    Occupancy and Proximity algorithms.

    Args:
        signal_frame: SignalFrame object (e.g. extracted by `read_signal_stream` function).

    Returns:
        A dictionary with the following fields:
        - **sequence_number** (*int*): sequence number of the input signal frame object.
        - **effectiveRange** (*int16*): effective range of the algorithm.
        - **powerPerBin** (*numpy.ndarray*): NumPy array of 2d points: [range in cm, power in dB].
        - **threshPerBin** (*numpy.ndarray*): NumPy array of 2d points: [range in cm, threshold in dB].
        - **range_bins_cm** (*numpy.ndarray*): NumPy array range bins in cm.
    """

    supported_nodes = [b'detector', b'detector_occupancy']
    for signal in sfu.get_supported_signals(signal_frame, [b'human_presence'], supported_nodes):
        range_array = sfu.get_array(signal, [b'range_lookup_table'])
        if range_array is None:
            return None

        power_array = sfu.get_array(signal, [b'power_per_bin_uint32'])
        if power_array is None:
            return None

        threshold_array = sfu.get_array(signal, [b'ThreshPerBin'])
        if threshold_array is None:
            return None

        effective_range_array = sfu.get_array(signal, [b'effective_range'])
        effective_range = effective_range_array.DataInt16AsNumpy()[0] if effective_range_array else 0

        range_data = range_array.DataInt16AsNumpy()
        power_data = power_array.DataUInt32AsNumpy()
        threshold_data = threshold_array.DataUInt16AsNumpy()

        power_points = []
        threshold_points = []

        for i in range(0, range_data.size):
            if effective_range and effective_range < range_data[i]:
                break

            # If there are two ranges with the same value take threshold and power for the second one.
            # Also skip ranges with zero threshold
            if i < range_data.size - 1 and range_data[i] == range_data[i + 1] or threshold_data[i] == 0:
                continue

            power_points.append([range_data[i], 10 * np.log10(power_data[i]) if power_data[i] else 0])
            threshold_points.append([range_data[i], 10 * np.log10(threshold_data[i])])

        detection_range_start = threshold_points[0][0]
        detection_range_end = effective_range if effective_range else range_data[-1]

        return {
            'sequence_number': int(signal_frame.SequenceNumber()),
            'effectiveRange': effective_range,
            'powerPerBin': np.array(power_points),
            'threshPerBin': np.array(threshold_points),
            'rangeBins_cm': np.linspace(detection_range_start, detection_range_end, len(threshold_points))
        }

    return None


# pylint: enable=too-many-locals
def get_distance(power_data, frames: int = 10):
    """
    Calculates distance to the closest target based on power data from `get_power` function.

    The distance is calculation is optimized for Proximity algorithm.

    Args:
        power_data: Dictionary produced by `get_power` function.
        frames: How many frames takes into accout.

    Returns:
        Distance in cm.
    """

    if power_data is None:
        return None

    if not hasattr(get_distance, 'distance_array'):
        get_distance.distance_array = collections.deque(maxlen=frames)

    power_per_bin = power_data['powerPerBin']
    threshold_per_bin = power_data['threshPerBin']
    range_bins = power_data['rangeBins_cm']

    if power_per_bin.size == 0:
        return None

    # find the first bin where power exceeds the threshold
    bins = np.asarray(power_per_bin[:, 1] > threshold_per_bin[:, 1]).nonzero()
    bins = np.array(bins).ravel()
    new_distance = range_bins[bins[0]] if len(bins) else 0

    get_distance.distance_array.append(new_distance)

    # take average of non-zero values
    non_zero_count = np.count_nonzero(get_distance.distance_array)
    return 0 if non_zero_count == 0 else round(np.sum(get_distance.distance_array) / float(non_zero_count))
